// Copyright 1998-2000 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.

#import <OmniSCSI/OSCSICDAudioStream.h>

NSString *OSCSICDAudioStreamInterruptedException = @"OSCSICDAudioStreamInterruptedException";

@interface OSCSICDAudioStream (PrivateAPI)
- (void) _readBlocks;
@end

@implementation OSCSICDAudioStream

- initWithAudioSource: (NSObject <OSCSICDRomAudioData> *) audioSource
          startingLBA: (OSCSILogicalBlockAddress) startingLBA
            endingLBA: (OSCSILogicalBlockAddress) endingLBA
    blocksPerTransfer: (OSCSILogicalBlockAddress) blocksPerTransfer;
{
    _audioSource       = [audioSource retain];
    _startingLBA       = startingLBA;
    _endingLBA         = endingLBA;
    _blocksPerTransfer = blocksPerTransfer;

    _currentLBA  = _startingLBA;

    return self;
}

- (void) dealloc;
{
    [_audioSource release];
    [_bufferQueue release];
    [super dealloc];
}

- (NSData *) nextAudioBlock;
{
    if (!_bufferQueue) {
        // Should we provide a way to control this parameter?
        _bufferQueue = [[OFQueue alloc] initWithCount: 256];

        [NSThread detachNewThreadSelector: @selector(_readBlocks) toTarget: self withObject: nil];
    }

    return [_bufferQueue dequeueShouldWait: YES];
}

@end


@implementation OSCSICDAudioStream (PrivateAPI)

- (void) _readBlocks;
{
    while (_currentLBA < _endingLBA) {
        NSAutoreleasePool        *pool;
        NSData                   *data;
        OSCSILogicalBlockAddress  blocksToRead;
        
        pool = [[NSAutoreleasePool alloc] init];

        blocksToRead = MIN(_blocksPerTransfer, _endingLBA - _currentLBA);
        data = [_audioSource readAudioDataAtLBA: _currentLBA
                                     blockCount: blocksToRead];
        if (!data) {
#warning We should return some sort of error condition to the owner of the stream
            NSLog(@"Failed to read audio data at LBA %d\n", _currentLBA);
            [pool release];
            break;
        } else if ([data length] != blocksToRead * OSCSI_CDDA_BYTES_PER_BLOCK) {
            NSLog(@"Read %d bytes, wanted %d\n", [data length], blocksToRead * OSCSI_CDDA_BYTES_PER_BLOCK);
        }
            
        if (![_bufferQueue enqueue: data shouldWait: NO]) {
            // This means that there will be an interruption in the streaming
            [NSException raise: OSCSICDAudioStreamInterruptedException
                        format: @"Unable to enqueue sound buffer -- read buffers quicker."];
        }
        
        _currentLBA += blocksToRead;
        [pool release];
    }

    [_bufferQueue close];
}

@end

