// Copyright 1998-2000 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.
//
// $Header: /Network/Source/CVS/OmniGroup/./Frameworks/OmniSCSI/OSCSICDProtocols.h,v 1.7 2000/09/08 19:18:35 bungi Exp $

#import <OmniFoundation/OmniFoundation.h>

#import <OmniSCSI/OSCSIUnit.h> // For OSCSILogicalBlockAddress
#import <OmniSCSI/FrameworkDefines.h>

@class OSCSICDTableOfContents;

OmniSCSI_EXTERN NSString *OSCSICDAudioUnitTableOfContentsChangedNotification;

// This should be used as the beginning of the play position
// to simply continue or start playing from the current position.
#define OSCSI_USE_CURRENT_PLAY_POSITION (0xffffffff)

#define OSCSI_CDDA_FRAMES_PER_SECOND (75)

typedef enum _OSCSICDAudioStatus {
    OSCSICDAudioUnknownStatus,
    OSCSICDAudioIdleStatus,
    OSCSICDAudioPlayingStatus,
    OSCSICDAudioPausedStatus,
    OSCSICDAudioScanningStatus
} OSCSICDAudioStatus;

@protocol OSCSICDRomAudioControl
- (void) updateStatus;

- (BOOL) mediaPresent;
- (void) unloadMedia;

- (OSCSICDTableOfContents *) tableOfContents;

- (void) startPlayingLBARange: (NSRange) lbaRange;
- (void) stopPlaying;
- (void) setAudioIsPaused: (BOOL) paused;

- (OSCSILogicalBlockAddress) currentLBA;
- (OSCSICDAudioStatus) currentStatus;

@end

@protocol OSCSICDRomAudioIdentification
- (NSString *) mediaCatalogueNumber;
- (NSString *) internationalStandardRecordingCodeForTrack: (unsigned char) trackNumber;
@end

@protocol OSCSICDRomAudioVolumeControl
// The percentage here should be a number between 0 and 1.
- (float) leftVolumePercentage;
- (float) rightVolumePercentage;
- (void) setLeftVolumePercentage: (float) leftPercentage
           rightVolumePercentage: (float) rightPercentage;
@end

@protocol OSCSICDRomAudioScan
- (void) startScanningFromLBA: (OSCSILogicalBlockAddress) lba
                      forward: (BOOL) forward;
- (void) stopScanningAndResumePlayingUntilLBA: (OSCSILogicalBlockAddress) lba;
@end

typedef struct _OSCSICDAudioSample {
    unsigned short left;
    unsigned short right;
} OSCSICDAudioSample;

#define OSCSI_CDDA_BYTES_PER_BLOCK   (2352)
#define OSCSI_CDDA_SAMPLES_PER_BLOCK (OSCSI_CDDA_BYTES_PER_BLOCK/sizeof(OSCSICDAudioSample))

@protocol OSCSICDRomAudioData
- (NSData *) readAudioDataAtLBA: (OSCSILogicalBlockAddress) lba
                     blockCount: (OSCSILogicalBlockAddress) blockCount;
// This will initiate or continue a reading stream.  If you don't call this
// often enough, the stream will be broken and an error may result.  Some
// CDROMs can recover, some can't.  The data returned as an array of OSCSICDAudioSample
// values.  NOTE: The data that comes off the disc is in little-endian format, but this
// method will automatically perform the byte swapping necessary if you are on a big-endian
// system.  You do not have to byte-swap the data yourself.
@end


typedef struct _OSCSIMinuteSecondsFrames {
    unsigned char minutes;
    unsigned char seconds;
    unsigned char frames;
} OSCSIMinuteSecondsFrames;


@protocol OSCSICDRecorder

- (unsigned short) maximumWriteSpeed;

- (void) beginWritingWithTestMode: (BOOL) testMode;
- (void) setWriteSpeed: (unsigned short) writeKBPerSecond;
// If writeKBPerSecond is zero, the maximum speed will be used.

- (void) setTableOfContents: (OSCSICDTableOfContents *) toc;

- (void) writeData: (NSData *) data atAddress: (OSCSILogicalBlockAddress) lba;

- (void) finishWriting;

@end

// LBA of 0 -> MSF 00:00:00
static inline OSCSIMinuteSecondsFrames OSCSIRelativeLogicalBlockAddressToMinutesSecondsFrames(OSCSILogicalBlockAddress lba)
{
    OSCSIMinuteSecondsFrames msf;

    msf.minutes = lba / (60 * OSCSI_CDDA_FRAMES_PER_SECOND);
    lba %= (60 * OSCSI_CDDA_FRAMES_PER_SECOND);

    msf.seconds = lba / OSCSI_CDDA_FRAMES_PER_SECOND;
    msf.frames = lba % OSCSI_CDDA_FRAMES_PER_SECOND;

    return msf;
}

// LBA of 0 -> MSF 00:02:00
static inline OSCSIMinuteSecondsFrames OSCSIAbsoluteLogicalBlockAddressToMinutesSecondsFrames(OSCSILogicalBlockAddress lba)
{
    return OSCSIRelativeLogicalBlockAddressToMinutesSecondsFrames(lba + 2 * OSCSI_CDDA_FRAMES_PER_SECOND);
}

// MSF of 00:00:00 -> LBA 0
static inline OSCSILogicalBlockAddress OSCSIMinutesSecondsFramesToRelativeLogicalBlockAddress(OSCSIMinuteSecondsFrames msf)
{
    return (msf.minutes * 60 * OSCSI_CDDA_FRAMES_PER_SECOND) + (msf.seconds * OSCSI_CDDA_FRAMES_PER_SECOND) + msf.frames;
}

// MSF of 00:02:00 -> LBA 0
static inline OSCSILogicalBlockAddress OSCSIMinutesSecondsFramesToAbsoluteLogicalBlockAddress(OSCSIMinuteSecondsFrames msf)
{
    return OSCSIMinutesSecondsFramesToRelativeLogicalBlockAddress(msf) - (2 * OSCSI_CDDA_FRAMES_PER_SECOND);
}
