// Copyright 1998-2000 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.

#import <OmniSCSI/OSCSIMinutesSecondsFramesFormatter.h>

#import <OmniSCSI/OSCSICDProtocols.h>
#import <Foundation/Foundation.h>

@implementation OSCSIMinutesSecondsFramesFormatter

- (void)setPerformsRelatveLBAConversion:(BOOL)value; { performsRelatveLBAConversion = value; }
- (BOOL)performsRelatveLBAConversion; { return performsRelatveLBAConversion; }

- (NSString *)stringForObjectValue:(id)object;
{
    OSCSIMinuteSecondsFrames msf;

    if (![object isKindOfClass:[NSNumber class]])
        return nil;

    if (performsRelatveLBAConversion)
        msf = OSCSIRelativeLogicalBlockAddressToMinutesSecondsFrames((OSCSILogicalBlockAddress)[(NSNumber *)object unsignedIntValue]);
    else
        msf = OSCSIAbsoluteLogicalBlockAddressToMinutesSecondsFrames((OSCSILogicalBlockAddress)[(NSNumber *)object unsignedIntValue]);

    return [NSString stringWithFormat:@"%02u:%02u.%02u", msf.minutes, msf.seconds, msf.frames];
}

- (BOOL)getObjectValue:(id *)obj forString:(NSString *)string errorDescription:(NSString **)error;
{
    NSArray *elements;
    int elementIndex;
    int numElements;
    OSCSIMinuteSecondsFrames msf;

    elements = [string componentsSeparatedByString:@":"];
    elementIndex = 0;
    numElements = [elements count];
    if (numElements > 1) {
        msf.minutes = (unsigned char)[[elements objectAtIndex:elementIndex] intValue];
        elementIndex++;
    }

    elements = [[elements objectAtIndex:elementIndex] componentsSeparatedByString:@"."];
    elementIndex = 0;
    numElements = [elements count];
    if (numElements > 0) {
        msf.seconds = (unsigned char)[[elements objectAtIndex:elementIndex] intValue];
        elementIndex++;
    }
    if (numElements > 1)
        msf.frames = (unsigned char)[[elements objectAtIndex:elementIndex] intValue];

    if (performsRelatveLBAConversion)
        *obj = [NSNumber numberWithUnsignedInt:(unsigned int)OSCSIMinutesSecondsFramesToRelativeLogicalBlockAddress(msf)];
    else
        *obj = [NSNumber numberWithUnsignedInt:(unsigned int)OSCSIMinutesSecondsFramesToAbsoluteLogicalBlockAddress(msf)];

    return YES;
}

@end
