// Copyright 1998-2000 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.

#import <OmniSCSI/OSCSIResponse.h>

#import <dev/scsireg.h>

static OSCSIResponse *_goodResponse = nil;

@implementation OSCSIResponse

+ (OSCSIResponse *) goodResponse;
{

    if (!_goodResponse) {
        _goodResponse = [[self alloc] initWithStatusByte: STAT_GOOD
                                            driverStatus: SR_IOST_GOOD
                                                senseKey: SENSE_NOSENSE
                                     additionalSenseCode: 0x0
                            additionalSenseCodeQualifier: 0x0
                                      bytesNotTransfered: 0];
    }

    return _goodResponse;
}

- initWithStatusByte: (unsigned int) statusByte
        driverStatus: (unsigned int) driverStatus
            senseKey: (unsigned int) senseKey
 additionalSenseCode: (unsigned int) additionalSenseCode
additionalSenseCodeQualifier: (unsigned int) additionalSenseCodeQualifier
  bytesNotTransfered: (int) bytesNotTransfered;
{
    _statusByte = statusByte;
    _driverStatus = driverStatus;
    _senseKey = senseKey;
    _additionalSenseCode = additionalSenseCode;
    _additionalSenseCodeQualifier = additionalSenseCodeQualifier;
    _bytesNotTransfered = bytesNotTransfered;

    return self;
}

- (BOOL) isError;
{
    if (self == _goodResponse)
        // short cicuit this one
        return NO;
    
    if (_statusByte != STAT_GOOD     && _statusByte != STAT_CONDMET &&
        _statusByte != STAT_INTMGOOD && _statusByte != STAT_INTMCONDMET)
        return YES;

    if (_driverStatus != SR_IOST_GOOD)
        return YES;
    
    if (_senseKey != SENSE_NOSENSE &&  _senseKey != SENSE_RECOVERED)
        return YES;
    
    return NO;
}

- (unsigned int) statusByte;
{
    return _statusByte;
}

- (unsigned int) driverStatus;
{
    return _driverStatus;
}

- (unsigned int) senseKey;
{
    return _senseKey;
}

- (unsigned int) additionalSenseCode;
{
    return _additionalSenseCode;
}

- (unsigned int) additionalSenseCodeQualifier;
{
    return _additionalSenseCodeQualifier;
}

- (int) bytesNotTransfered;
{
    return _bytesNotTransfered;
}

- (NSString *) errorString;
{
#warning Later, we should take the ASCII list of errors and put them in a plist so that we can have a meaningful string
    return [NSString stringWithFormat: @"sense key = %02XH, additional sense code = %02XH, additional sense code qualifier = %02XH", _senseKey, _additionalSenseCode, _additionalSenseCodeQualifier];
}

- (BOOL) isMediaMissingError;
{
    if (_additionalSenseCode == 0x3A)
        // MEDIUM NOT PRESENT;
        return YES;

    if (_additionalSenseCode == 0x04 && _additionalSenseCodeQualifier == 0x01)
        // LOGICAL UNIT IS IN PROCESS OF BECOMING READY
        return YES;

    if (_additionalSenseCode == 0x28 && _additionalSenseCodeQualifier == 0x00)
        // NOT READY TO READY CHANGE, MEDIUM MAY HAVE CHANGED
        return YES;

    return NO;
}

@end

