// Copyright 1998-2000 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.
//
// $Header: /Network/Source/CVS/OmniGroup/./Frameworks/OmniSCSI/OSCSITarget.h,v 1.13 2000/09/08 19:18:36 bungi Exp $

#import <OmniFoundation/OmniFoundation.h>

#import <OmniSCSI/OSCSIUnit.h> // For OSCSIUnitNumberType
#import <OmniSCSI/OSCSIBus.h>  // For OSCSIBusNumberType

@class OSCSIBus;

typedef unsigned long long OSCSITargetNumberType;

// If this is used when initializing the OSCSITarget, it will only
// use the raw device (instead of the generic device).  This is
// useful when you don't want to scan the bus to find the SCSI
// target or when the device is an ATAPI device (and doesn't have
// a bus/target/unit).
#define OSCSI_INVALID_TARGET_NUMBER ((OSCSITargetNumberType)-1)

@interface OSCSITarget : OFObject
{
    NSString              *_rawDeviceName;
    OSCSIBusNumberType     _busNumber;
    OSCSITargetNumberType  _targetNumber;
    NSArray               *_units;
    
}

- initWithBusNumber: (OSCSIBusNumberType) busNumber
       targetNumber: (OSCSITargetNumberType) targetNumber;

// This initializer can be used when the raw device name is known before
// the device is queries.  The only currently known case where this is
// valid/useful is when scanning for ATAPI devices.
- initWithBusNumber: (OSCSIBusNumberType) busNumber
       targetNumber: (OSCSITargetNumberType) targetNumber
      rawDeviceName: (NSString *) rawDeviceName;

// After the target is initialized and the type of target is known
// this must be set to the path to the raw Unix device.
- (void) setRawDeviceName: (NSString *) rawDeviceName;
- (NSString *) rawDeviceName;

- (OSCSIBusNumberType) busNumber;
- (OSCSITargetNumberType) targetNumber;

- (NSArray *) units;
- (OSCSIUnit *) unitWithUnitNumber: (OSCSIUnitNumberType) unitNumber;


@end
