// Copyright 1997-1999 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.
//
// $Header: /Network/Source/CVS/OmniGroup/./Frameworks/OmniSCSI/cdburn/cdburn_main.m,v 1.6 2000/02/21 18:04:30 bungi Exp $


/*
 *
 * This doesn't do anything really useful yet -- it is still under development.
 *
 *
 */


#import <Foundation/Foundation.h>
#import <OmniSCSI/OmniSCSI.h>
#import <OmniSoundKit/OSKSampleSource.h>
#import <OmniSoundKit/OSKSNDFileSampleSource.h>
#import <OmniBase/OmniBase.h>

int main (int argc, const char *argv[])
{
    NSAutoreleasePool           *pool = [[NSAutoreleasePool alloc] init];
    OSCSITarget                 *target;
    OSCSIUnit                   *unit;
    OSCSICDTableOfContents      *toc;
    id <OSCSICDRecorder>         recorder;
    unsigned int                 trackNumber, trackCount;
    OSCSICDTableOfContentsEntry *tocEntries;
    NSMutableArray              *sampleSources;
    OSCSILogicalBlockAddress     lba = 0;
    NSTimeInterval               timeInterval;
    
    [OBPostLoader processClasses];

    if (argc < 3) {
        fprintf(stderr, "usage: %s raw-scsi-device file1.snd ... fileN.snd\n", argv[0]);
        exit(1);
    }

    NS_DURING{
        target = [[OSCSITarget alloc] initWithBusNumber: OSCSI_INVALID_BUS_NUMBER
                                           targetNumber: OSCSI_INVALID_TARGET_NUMBER
                                          rawDeviceName: [NSString stringWithCString: argv[1]]];
        unit = [target unitWithUnitNumber: 0];
        [unit open];
    } NS_HANDLER {
        NSLog(@"Error connecting to requested unit: %@", localException);
        exit(1);
    } NS_ENDHANDLER;

    printf("Selected unit: %s\n", [[[unit debugDictionary] description] cString]);

    if (![unit isKindOfClass: [OSCSI2CDRomUnit class]]) {
        fprintf(stderr, "The specified unit is not a CD-ROM device.\n");
        exit(1);
    }

    if (!(recorder = [(OSCSI2CDRomUnit *)unit recorderUnit])) {
        fprintf(stderr, "The specified unit cannot record CD-ROMs.\n");
        exit(1);
    }

    // skip the non-file arguments
    argv += 2;
    argc -= 2;

    sampleSources = [[NSMutableArray alloc] init];
    trackCount = argc;
    timeInterval = [[NSDate distantFuture] timeIntervalSinceReferenceDate];
    
    // Allocate slots for all the tracks, plus the leadout
    tocEntries = NSZoneMalloc(NULL, sizeof(*tocEntries) * (trackCount + 1));
    for (trackNumber = 0; trackNumber < trackCount; trackNumber++) {
        NSString                 *fileName;
        OSKSampleIndex            sampleCount;
        OSKSampleSource          *sampleSource;

        
        OSCSILogicalBlockAddress  blocksInSource;
        
        fileName = [NSString stringWithCString: argv[trackNumber]];
        sampleSource = [OSKSNDFileSampleSource sampleSourceWithSNDFileAtPath: fileName
                                                             framesPerBuffer: 64 * OSCSI_CDDA_SAMPLES_PER_BLOCK
                                                          sequentialReadOnly: YES];

        [sampleSources addObject: sampleSource];

        tocEntries[trackNumber].trackNumber = trackNumber + 1; // CD tracks are 1-based indexing!
        tocEntries[trackNumber].startingLBA = lba;

        // We must write even sized blocks -- we'll pad files out with zeros
        sampleCount = [sampleSource sampleCount];
        blocksInSource = sampleCount / OSCSI_CDDA_SAMPLES_PER_BLOCK;
        if (sampleCount % OSCSI_CDDA_SAMPLES_PER_BLOCK)
            blocksInSource++;

        lba += blocksInSource;
    }

    // Add leadout
    tocEntries[trackCount].trackNumber = OSCSI_LEAD_OUT_TRACK;
    tocEntries[trackCount].startingLBA = lba;
    toc = [[OSCSICDTableOfContents alloc] initWithTableOfContentsDescriptions: tocEntries
                                                                        count: trackCount + 1];

    NSLog(@"toc = %@", toc);

    NS_DURING {
        [recorder beginWritingWithTestMode: YES];
        [recorder setWriteSpeed: 0]; // max speed
        [recorder setTableOfContents: toc];
    } NS_HANDLER {
        NSLog(@"Exception raised: %@", localException);
        [(id)recorder requestSense];
        exit(1);
    } NS_ENDHANDLER;

    // Write the actual data.  This will put NO pauses between tracks.
    lba = 0;
    for (trackNumber = 0; trackNumber < trackCount; trackNumber++) {
        OSKSampleSource          *sampleSource;
        OSKSampleIndex            sampleIndex, sampleCount;
        
        sampleSource = [sampleSources objectAtIndex: trackNumber];
        sampleIndex = 0;
        sampleCount = [sampleSource sampleCount];
        while (sampleIndex < sampleCount) {
            NSAutoreleasePool        *pool;
            NSData                   *buffer;
            unsigned int              bufferLength;
            OSCSILogicalBlockAddress  blocksInBuffer;
            OSKSampleIndex            samplesInBuffer;

            pool = [[NSAutoreleasePool alloc] init];
            
            buffer = [sampleSource sampleBufferStartingAtSampleIndex: sampleIndex
                                                  beforeTimeInterval: timeInterval];
            bufferLength = [buffer length];

            samplesInBuffer = bufferLength / sizeof(OSCSICDAudioSample);
            if (samplesInBuffer % OSCSI_CDDA_SAMPLES_PER_BLOCK) {
                // Round the data up to a multiple of the block size
                buffer = [[buffer mutableCopy] autorelease];

                samplesInBuffer += OSCSI_CDDA_SAMPLES_PER_BLOCK;
                bufferLength = samplesInBuffer * OSCSI_CDDA_SAMPLES_PER_BLOCK;
                [(NSMutableData *)buffer setLength: bufferLength];
            }
            blocksInBuffer  = bufferLength / OSCSI_CDDA_BYTES_PER_BLOCK;

            NS_DURING {
                [recorder writeData: buffer atAddress: lba];
            } NS_HANDLER {
                [(id)recorder requestSense];
                //[localException raise];
            } NS_ENDHANDLER;

            [pool release];
            
            sampleIndex += samplesInBuffer;
            lba += blocksInBuffer;
        }
    }

    [recorder finishWriting];
    [unit eject];

    [pool release];
    exit(0);       // insure the process exit status is 0
    return 0;      // ...and make main fit the ANSI spec.
}
