// Copyright 1997-1999 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.
//
// $Header: /Network/Source/CVS/OmniGroup/./Frameworks/OmniSCSI/cdplay/cdplay_main.m,v 1.8 1999/06/17 16:42:44 bungi Exp $

#import <Foundation/Foundation.h>
#import <OmniSCSI/OmniSCSI.h>
#import <OmniBase/OmniBase.h>

void main (int argc, const char *argv[])
{
    NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
    OSCSITarget   *target;
    OSCSIUnit     *unit;
    unsigned int  startingLBA, lbaCount;
    NSRange       range;
    OSCSICDTableOfContents *toc;
    
    id <NSObject, OSCSICDRomAudioControl> cdAudio;
    
    [OBPostLoader processClasses];

    if (argc != 3 && argc != 4) {
        fprintf(stderr, "usage: %s scsi-raw-device starting-logical-block-address [logical-block-count]\n", argv[0]);
        exit(1);
    }

    startingLBA = atoi(argv[2]);
    
    NS_DURING{
        target = [[OSCSITarget alloc] initWithBusNumber: OSCSI_INVALID_BUS_NUMBER
                                           targetNumber: OSCSI_INVALID_TARGET_NUMBER
                                          rawDeviceName: [NSString stringWithCString: argv[1]]];
        unit = [target unitWithUnitNumber: 0];
        [unit open];
    } NS_HANDLER {
        NSLog(@"Error connecting to requested unit: %@", localException);
        exit(1);
    } NS_ENDHANDLER;

    if (![unit conformsToProtocol: @protocol(OSCSICDRomAudioControl)]) {
        fprintf(stderr, "The specified unit is not a CD Audio device.\n%s\n",
                [[[unit debugDictionary] description] cString]);
        exit(1);
    }

    printf("Selected unit: %s\n", [[[unit debugDictionary] description] cString]);
           
    cdAudio = (id <NSObject, OSCSICDRomAudioControl>) unit;
    [cdAudio updateStatus];

    toc = [cdAudio tableOfContents];
    
    // Print out the toc
    printf("TOC: %s\n", [[[toc debugDictionary] description] cString]);

    if (argc == 4)
        lbaCount = atoi(argv[3]);
    else
        lbaCount = [toc startingLBAForTrackNumber: OSCSI_LEAD_OUT_TRACK] - startingLBA;

    // Play the requested audio.
    range = NSMakeRange(startingLBA, lbaCount);
    [cdAudio startPlayingLBARange: range];
        
    [pool release];
    exit(0);       // insure the process exit status is 0
}
